#include <tiny26.h>          //  1 
#include <delay.h>           // 

#define MUTE      0xC0       //   
#define SPEED     10         //    

#define enc_A1    PINB.5     //   
#define enc_B1    PINB.6     //   
#define enc_E1    PINB.4     //   
                    
#define enc_A2    PINA.4     //   
#define enc_B2    PINA.5     //   
#define enc_E2    PINA.3     //   

#define PCLOCK    PORTA.7    //  
#define SCL       DDRA.7     //   7448
#define PDATA     PORTA.6    //  
#define SDA       DDRA.6     //   7448

#define LED_1     PORTA.0    //  1 
#define LED_2     PORTA.1    //  2 
#define LED_3     PORTA.2    //  3 
#define LED_4     PORTB.0    //  4 
#define LED_5     PORTB.1    //  5 
#define LED_6     PORTB.2    //  6 
#define LED_7     PORTB.3    //  7 

void EncoderScan1 (void);           //  1
void EncoderScan2 (void);            //  2
void LED_bar  (unsigned char level); //  
void TDA_send (unsigned char value); //   
void TWI_send (unsigned char data);  //    i2c
void TWI_start(void);                //   i2c
void TWI_stop (void);                //   i2c
void save_vol (void);                //   

unsigned char vol;           //  
unsigned char trigger;       //  
unsigned char timer;         //   
unsigned char mute;          //   

eeprom unsigned char e_vol; //  
eeprom unsigned char e_mute;//          

unsigned char enc_state1;    //    1
unsigned char enc_state2;    //    2
unsigned char enc_data1;     //    1
unsigned char enc_data2;     //    2
  
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
 {
  timer++;           //   
  if(timer>20)       //   600 ?
   {
    if(e_vol!=vol)   //   
    {e_vol=vol;}     //  , 
    delay_ms(5);     //   
    
    if(e_mute!=mute) //   
    {e_mute=mute;}   //  , 
    delay_ms(5);     //   
    
    timer=0;         //  
    #asm("cli")      //    
   }                 //            
 } 

//=================== Encoder ==================
void tda_plavno (unsigned char direction)
 {
  unsigned char steps;  //  
  unsigned char tmp=80; //  
  
  if(direction)         // 1 - 
   {
    steps=80;           //   
    for(tmp-=vol; tmp>0; tmp--) //  +
     {
      steps--;          //  
      TDA_send(steps);  //  
      delay_ms(SPEED);  //   
      LED_bar(steps);   //  
     }
   }
  else                  // 0 - 
   {
    steps=vol;          //   
    for(tmp-=vol; tmp>0; tmp--) //  -
     {
      steps++;          //  
      if(steps!=80) TDA_send(steps);  //    
      delay_ms(SPEED);  //   
      LED_bar(steps);   //  
     }
    TDA_send(MUTE);     //  mute
   }  

 }

//=================== Encoder ==================
void main(void)
 {  
  DDRA=0b00000111;      //  
  DDRB=0b10001111;      //  
  
  TDA_send(MUTE);       //  mute
  
  vol=e_vol;            //  
  if(vol>80)            //   
  {e_vol=80; vol=80;}   //  
  delay_ms(5);          //  
  
  mute=e_mute;          //  
  if(mute>1)            //   
  {mute=0; e_mute=0;}   //  
  delay_ms(5);          //  
  
  if(!mute && vol!=80)  //    0, ..
  {tda_plavno(1);}      //    
  
  TCCR0=0x03;           //   
  TIMSK=0x02;           //     
  
  while(1)
   {
    if(!enc_E1 && trigger==0) {trigger=1;} //    1
    if( enc_E1 && trigger==1)  //    1
     {trigger=0;
      mute^=1;                 //  
          
      if(mute){tda_plavno(0);} //  
      else    {tda_plavno(1);} //   
      
      save_vol();              //  
     }
    if(!enc_E2 && trigger==0) {trigger=2;} //    2
    if( enc_E2 && trigger==2)  //    2
     {trigger=0; 
      mute^=1;                 //  
          
      if(mute){tda_plavno(0);} //  
      else    {tda_plavno(1);} //  
      
      save_vol();              //  
     }
                        
    EncoderScan1();   //   1 
    EncoderScan2();   //   2 
    
    if(enc_data1==1 || enc_data2==1)  // 1 - 
     {
      enc_data1=0; //    
      enc_data2=0; //    
      
      if(vol<80 && !mute)   //   mute
       {
        vol++;     //  

        if(vol==80) {TDA_send(MUTE);}  //  80,   
        else        {TDA_send(vol);}   //   
        LED_bar(vol);  //  
         
        save_vol();//  
       }
     } 
    if(enc_data1==2 || enc_data2==2)  // 2 - 
     {
      enc_data1=0; //    
      enc_data2=0; //    
      
      if(mute) vol=80; //     
      
      if(vol>0)    //    
       {
        vol--;     //   
        
        if(vol<80) mute=0; //     +
        
        TDA_send(vol); //    I2C
        LED_bar(vol);  //  
          
        save_vol();    // 
       }
     } 

     delay_us(300);         //   
   }
 }

void EncoderScan1 (void)  //  1
 {
  unsigned char enc_temp=0;         //
  
  if(enc_A1){enc_temp|=(1<<0);}      //  
  if(enc_B1){enc_temp|=(1<<1);}      //  

  if(enc_state1==0)               // 
   {
    if(enc_temp==2) enc_data1=2;
    if(enc_temp==1) enc_data1=1; 
   }
  if(enc_state1==3)               // 
   {
    if(enc_temp==1) enc_data1=2;
    if(enc_temp==2) enc_data1=1; 
   }
   
  enc_state1 = enc_temp;  //    
 } 
void EncoderScan2 (void)  //  2
 {
  unsigned char enc_temp=0;         //
  
  if(enc_A2){enc_temp|=(1<<0);}      //  
  if(enc_B2){enc_temp|=(1<<1);}      //  

  if(enc_state2==0)               // 
   {
    if(enc_temp==2) enc_data2=2;
    if(enc_temp==1) enc_data2=1; 
   }
  if(enc_state2==3)               // 
   {
    if(enc_temp==1) enc_data2=2;
    if(enc_temp==2) enc_data2=1; 
   }
   
  enc_state2 = enc_temp;  //    
 }
void LED_bar(unsigned char level)
 {
  unsigned char bar[8]={0,0,0,0,0,0,0,0}; //  
  unsigned char n=0;  //            
  
  if(level<=10 && level>=0) {n=1;} // 7 
  if(level<=20 && level>10) {n=2;} // 6 
  if(level<=30 && level>20) {n=3;} // 5 
  if(level<=40 && level>30) {n=4;} // 4 
  if(level<=50 && level>40) {n=5;} // 3 
  if(level<=60 && level>50) {n=6;} // 2 
  if(level<=70 && level>60) {n=7;} // 1 
  if(level<=80 && level>70) {n=8;} //  
     
  for(; n<8; n++) {bar[n]=1;}      // 
  
  //  
  LED_1=bar[7];LED_2=bar[6];LED_3=bar[5];LED_4=bar[4];
  LED_5=bar[3];LED_6=bar[2];LED_7=bar[1]; 
 } 
void save_vol (void) //  
 {
  TCNT0=0x00;   //   
  timer=0;      //   
  #asm("sei")  //   
 }
void TWI_start (void)      //   
 {
  #define WHI      0   //   i2c
  #define WLO      1   //    i2c
  PDATA=0;             // SDA   
  PCLOCK=0;            // SCL  
  SCL=WHI;             // 
  #asm("nop") 
  #asm("nop")
  SDA=WLO;             // 
  #asm("nop") 
  #asm("nop")
  SCL=WLO;             // 
  #asm("nop")
 }
void TWI_send (unsigned char data) //  
 {
  unsigned char tmp;     //
  
  for(tmp=0; tmp<8; tmp++)   // 
   {
    if(data & 128)     // 
     {SDA=WHI;}        //  = 1
    else
     {SDA=WLO;}        //  = 0  
     
    #asm("nop")  #asm("nop")
          
    SCL=WHI;           // 
    data=data<<1;      // 
    SCL=WLO;           // 
   }
    
  #asm("nop")  #asm("nop")
  
  //  Acknowledge  
  SDA=WHI;  #asm("nop")  #asm("nop")
  SCL=WHI;  #asm("nop")  #asm("nop")
  SCL=WLO;  #asm("nop")  #asm("nop")
 } 
void TWI_stop (void)   //   
 {
  PDATA=0;             // SDA   
  PCLOCK=0;            // SCL  
  SDA=WLO;             // 
  #asm("nop") 
  #asm("nop")
  SCL=WHI;             //  
  #asm("nop") 
  #asm("nop")
  SDA=WHI;             //  
  #asm("nop")
 } 
void TDA_send (unsigned char value)
 {
  TWI_start();       // 
  TWI_send(0x88);    //
  TWI_send(0x10);    //
  TWI_send(value);   //1- 
  TWI_send(value);   //2- 
  TWI_send(value);   //3- 
  TWI_send(value);   //4- 
  TWI_send(value);   //5- 
  TWI_send(value);   //6- 
  TWI_stop();        //  
 }
 